/********************************************************************
*	THE USE BY YOU OF THIS SOFTWARE IS GOVERNED IN ALL
*	RESPECTS BY THE MADGE NETWORKS LTD OPEN SOURCE LICENSE.		
*	WE ADVISE YOU TO READ THE TERMS OF THE OPEN SOURCE LICENSE
*	WHICH CAN BE FOUND WITHIN THE LICENSE.TXT FILE WHICH IS PART OF
*	THE SOFTWARE OR BY VISITING THE URL
*	http://www.madge.com/Connect/Downloads/Software/.		
*	MADGE NETWORKS LTD RESERVES THE RIGHT TO SEEK REDRESS FOR
*	ANY VIOLATION OF THE LICENSE'S TERMS					
*********************************************************************/

/*****************************************************************************
*
* details.c 
*
* Madge Token Ring Device Driver for Linux 2.0.x.
*
* Installation Program - Show Adapter Details dialog box.
*
* Copyright (c) Madge Networks Ltd 1998
*
* PVCS Version History:
*
*  $Id: details.c,v 1.1.1.1 2006/04/27 15:41:46 br Exp $
*  $Revision: 1.1.1.1 $
*  $Date: 2006/04/27 15:41:46 $
*  $Author: br $
*    
*****************************************************************************/
/* Please refer to the LICENSE file supplied with this source code */

#include <newt.h>
#include <stdlib.h>
#include <stdio.h>
#include "madgepci.h"
#include "bia.h"

#define HEX(c)	( (c>='0' && c<'9') ? c - '0' : ((c >= 'a' && c <='f') ? c - 'a' + 10 : c - 'A' + 10))

#define 	RS_AUTO		1
#define		RS_4M		3
#define		RS_16M		4
#define		RS_100M		5

char * madge_ringspeed_to_string(int rs)
{
	switch(rs)
	{
		case RS_AUTO: 	return "Auto";
		case RS_4M:		return "4 Mbps";
		case RS_16M:	return "16 Mbps";
		case RS_100M:	return "100 Mbps";
	}
	return "Unknown";
}

#define UNKNOWN	0
#define DTR		2
#define CLASSIC	1
#define SINGLE	3

char * madge_openmode_to_string(int om)
{
	switch(om)
	{
		case DTR:		return "DTR";
		case CLASSIC:	return "Classic";
		case SINGLE:	return "Single Station";
	}
	return "Unknown";
}

void GetAdvancedParameters(pci_device * dev)
{
	newtComponent AdvancedParametersForm,ok,cancel,answer,mfs;

	newtComponent rsauto,s4,s16,s100,dtr,classic,both;
	int col=3,row=1;
	int done = 0;
	int winsize = 14;
	newtComponent TXSlots,RXSlots;
	char *pTXSlots;
	char *pRXSlots;
	char *pMaxFrameSize;
	char buffer[15];
	winsize+=7;
		
	newtCenteredWindow(40,winsize,"Advanced Parameters");
	AdvancedParametersForm = newtForm(NULL,NULL,0);

	/** TX Slots **/

	newtFormAddComponent(AdvancedParametersForm,
		newtLabel(col,row,"TX Slots "));

	sprintf(buffer,"%d",dev->info.txslots);
	TXSlots = newtEntry(col+11,row,
							buffer,
							14,&pTXSlots,NEWT_FLAG_RETURNEXIT);

	newtFormAddComponent(AdvancedParametersForm,TXSlots);
	row++;

	/** RX Slots **/

	newtFormAddComponent(AdvancedParametersForm,
		newtLabel(col,row,"RX Slots "));

	sprintf(buffer,"%d",dev->info.rxslots);
	RXSlots = newtEntry(col+11,row,
							buffer,
							14,&pRXSlots,NEWT_FLAG_RETURNEXIT);

	newtFormAddComponent(AdvancedParametersForm,RXSlots);
	row+=2;

	/** DTR, Classic, Both **/
	newtFormAddComponent(AdvancedParametersForm,
		newtLabel(col,row++,"Open Mode"));

	both    = newtRadiobutton(col+5,row++,"Both          ",dev->info.dtr == 0  ? 1 : 0,NULL);
	dtr     = newtRadiobutton(col+5,row++,"DTR Only      ",dev->info.dtr == 1  ? 1 : 0,both); 
	classic = newtRadiobutton(col+5,row++,"Classic Only  ",dev->info.dtr == 2  ? 1 : 0,dtr);
	newtFormAddComponents(AdvancedParametersForm,both,dtr,classic,NULL);
	row++;
	
	/** Ring Speed **/
	
	newtFormAddComponent(AdvancedParametersForm,
		newtLabel(col,row++,"Ring Speed"));

	rsauto  = newtRadiobutton(col+5,row++,"Auto          ",dev->info.ringspeed == 0   ? 1 : 0,NULL); 
	s4      = newtRadiobutton(col+5,row++,"4 Mbps        ",dev->info.ringspeed == 4   ? 1 : 0,rsauto); 
	s16     = newtRadiobutton(col+5,row++,"16 Mbps       ",dev->info.ringspeed == 16  ? 1 : 0,s4);
	s100    = newtRadiobutton(col+5,row++,"100 Mbps      ",dev->info.ringspeed == 100 ? 1 : 0,s16);
	newtFormAddComponents(AdvancedParametersForm,rsauto,s4,s16,s100,NULL);
	row++;

	newtFormAddComponent(AdvancedParametersForm,
		newtLabel(col,row,"Max Frame Size "));
	
	sprintf(buffer,"%d",dev->info.max_frame_size);
	mfs = newtEntry(col+16,row,buffer,14,&pMaxFrameSize,NEWT_FLAG_RETURNEXIT);

	newtFormAddComponent(AdvancedParametersForm,mfs);	
	row+=2;
	
	/** Buttons **/	
	
	col = 12;
	
	ok = newtButton(col,row,"OK");
	cancel = newtButton(col+8,row,"Cancel");

	newtFormAddComponents(AdvancedParametersForm,ok,cancel,NULL);
	
	do {
		answer = newtRunForm(AdvancedParametersForm);
		if (answer == ok)
		{
			if (newtCheckboxGetValue(rsauto) == '*')
				dev->info.ringspeed = 0;
			else if (newtCheckboxGetValue(s4) == '*')
				dev->info.ringspeed = 4;
			else if (newtCheckboxGetValue(s16) == '*')
				dev->info.ringspeed = 16;
			else if (newtCheckboxGetValue(s100) == '*')
				dev->info.ringspeed = 100;

			if (newtCheckboxGetValue(both) == '*')
				dev->info.dtr = 0;
			else if (newtCheckboxGetValue(dtr) == '*')
				dev->info.dtr = 1;
			else if (newtCheckboxGetValue(classic) == '*')
				dev->info.dtr = 2;

			dev->info.rxslots = atoi(pRXSlots);
			dev->info.txslots = atoi(pTXSlots);
			dev->info.max_frame_size = atoi(pMaxFrameSize);
			done = 1;
		}
		else if (answer==cancel)
		{
			done = 1;
		}
	} while(!done);

	newtPopWindow();
	newtFormDestroy(AdvancedParametersForm);
}

void DisplayAdapterStatistics(pci_device * dev)
{
	newtComponent form,ok,answer;
	char	Buffer[80];
	int row = 1;
	int col = 3;
	int winsize = 0;
	int done = 0;
	int dobytes;
	
	dobytes = (dev->stats.rx_bytes != 0 || dev->stats.rx_packets == 0);
		
	/*
	 * Calculate the number of rows needed to display all the adapter
	 * information
	 */
	
	winsize = 3+10+2;
	if (dobytes) winsize+=2;
	
	newtCenteredWindow(40,winsize,"Adapter Statistics");
	form = newtForm(NULL,NULL,0);

	sprintf(Buffer,"Ring Speed    : %s",madge_ringspeed_to_string(dev->info.open_ring_speed));
	newtFormAddComponent(form,newtLabel(col,row++,Buffer));
	
	sprintf(Buffer,"Open Mode     : %s",madge_openmode_to_string(dev->info.mode));
	newtFormAddComponent(form,newtLabel(col,row++,Buffer));
	

	sprintf(Buffer,"RX Statistics");
	newtFormAddComponent(form,newtLabel(col,row++,Buffer));

	if (dobytes)
	{
		sprintf(Buffer,"   Bytes      : %ld",dev->stats.rx_bytes);
		newtFormAddComponent(form,newtLabel(col,row++,Buffer));
	}
	
	sprintf(Buffer,"   Frames     : %ld",dev->stats.rx_packets);
	newtFormAddComponent(form,newtLabel(col,row++,Buffer));
	sprintf(Buffer,"   Errors     : %ld",dev->stats.rx_errors);
	newtFormAddComponent(form,newtLabel(col,row++,Buffer));

	row++;

	sprintf(Buffer,"TX Statistics");
	newtFormAddComponent(form,newtLabel(col,row++,Buffer));

	if (dobytes)
	{
		sprintf(Buffer,"   Bytes      : %ld",dev->stats.tx_bytes);
		newtFormAddComponent(form,newtLabel(col,row++,Buffer));
	}

	sprintf(Buffer,"   Frames     : %ld",dev->stats.tx_packets);
	newtFormAddComponent(form,newtLabel(col,row++,Buffer));
	sprintf(Buffer,"   Errors     : %ld",dev->stats.tx_errors);
	newtFormAddComponent(form,newtLabel(col,row++,Buffer));

	col = 16;row++;

	ok = newtButton(col,row,"OK");

	newtFormAddComponents(form,ok,NULL);

	do {
		answer = newtRunForm(form);
		if (answer == ok)
		{
			done = 1;
		}
	} while(!done);


	newtFormDestroy(form);
	newtPopWindow();

}


void DisplayAdapterInfo(pci_device * dev)
{
	newtComponent form,text,ok,answer,advanced,statistics;
	newtGrid	grid,buttons;
	char	Buffer[256],tmpbia[16],tmplaa[16];
	int done = 0;
	int lines = 5;
	char * p;
	
	grid = newtCreateGrid(1,2);

	p = Buffer;
	
	p+= sprintf(p,		"Adapter       : %s\n",
						dev->cardid->description);

	if (dev->info.busdevfn)
	{
		p+=sprintf(p,"Bus Number    : %X\n"
    				 "Device Number : %X\n",
					PCI_BUS(dev->info.busdevfn),
					PCI_SLOT(dev->info.busdevfn & 0xFF));
		lines+=2;
	}    				 

	p+=sprintf(p,	"IO Base       : 0x%04X\n"
					"IRQ           : %d\n"
					"BIA           : %s\n"
                    "Current       : %s\n",
					dev->info.iobase,
					dev->info.irq,
					madge_bia_as_string(dev->info.bia,tmpbia),
					madge_bia_as_string(dev->info.laa,tmplaa)
					);

	text = newtTextbox(0,0,60,lines,0);
	newtTextboxSetText(text,Buffer);
	newtGridSetField(grid,0,0,NEWT_GRID_COMPONENT,
						text,
						0,0,0,0,NEWT_ANCHOR_LEFT,0);

	if (dev->info.state == 10 /* MTOK_STATE_OPEN */)
		buttons = newtButtonBar("Close",&ok,"Advanced",&advanced,"Open Info",&statistics,NULL);
	else
		buttons = newtButtonBar("Close",&ok,"Advanced",&advanced,NULL);
	
	newtGridSetField(grid,0,1,NEWT_GRID_SUBGRID,buttons,0,1,0,0,0,0);


	form = newtForm(NULL,NULL,0);
	newtGridAddComponentsToForm(grid,form,1);
	newtGridWrappedWindow(grid,"Adapter Details");
	newtGridFree(grid,1);

	do {
		answer = newtRunForm(form);
		if (answer == ok)
		{
			done = 1;
		}
		else if (answer==advanced)
		{
			GetAdvancedParameters(dev);
			newtDrawForm(form);
			newtRefresh();
			done = 0;
		} else if (answer == statistics)
		{
			DisplayAdapterStatistics(dev);
			newtDrawForm(form);
			newtRefresh();
		}
	} while(!done);
	
	newtFormDestroy(form);
	newtPopWindow();
}
