/********************************************************************
*	THE USE BY YOU OF THIS SOFTWARE IS GOVERNED IN ALL
*	RESPECTS BY THE MADGE NETWORKS LTD OPEN SOURCE LICENSE.		
*	WE ADVISE YOU TO READ THE TERMS OF THE OPEN SOURCE LICENSE
*	WHICH CAN BE FOUND WITHIN THE LICENSE.TXT FILE WHICH IS PART OF
*	THE SOFTWARE OR BY VISITING THE URL
*	http://www.madge.com/Connect/Downloads/Software/.		
*	MADGE NETWORKS LTD RESERVES THE RIGHT TO SEEK REDRESS FOR
*	ANY VIOLATION OF THE LICENSE'S TERMS					
*********************************************************************/

/*****************************************************************************
*
* main.c 
*
* Madge Token Ring Device Driver for Linux.
*
* Installation Code.
*
* Copyright (c) Madge Networks Ltd 1998-1999
*
* PVCS Version History:
*
*  $Id: main.c,v 1.1.1.1 2006/04/27 15:41:46 br Exp $
*  $Revision: 1.1.1.1 $
*  $Date: 2006/04/27 15:41:46 $
*  $Author: br $
*    
*****************************************************************************/
/* Please refer to the LICENSE file supplied with this source code */

#include <linux/config.h>
#include <stdlib.h>
#include <unistd.h>
#include <stdio.h>
#include <string.h>
#include <signal.h>
#include <sys/utsname.h>
#include <sys/time.h>
#include <sys/stat.h>
#include <net/if.h>

#include "install.upd"
#include "madgepci.h"
#include "bia.h"

char mtok_mver_title[] = "VeRsIoN=" MADGE_PROBE_PRODUCT_NAME; 
char mtok_mver_copyright[] = "VeRsIoN=" MADGE_PROBE_PRODUCT_COPYRIGHT;
char mtok_mver_platform[] = "VeRsIoN=PLATFORM: " MADGE_PROBE_PRODUCT_PLATFORM;
char mtok_mver_version[61] = "VeRsIoN=VERSION: v" MADGE_PROBE_VERSION;
char mtok_mver_product[] = "VeRsIoN=PRODUCT-ID: " MADGE_PROBE_PRODUCT_ID;
char mtok_crcp[] = "MaDgE=XXXX";


void generate_conf_modules(char * filename, pci_device * head);
void DisplayAdapterInfo(pci_device * dev);

/*
 * UI Stuff
 */
#include <newt.h>

#ifdef NEWNEWT
void suspend(void * data)
#else
void suspend(void)
#endif
{
	newtSuspend();
	raise(SIGTSTP);
	newtResume();
}
            
void FillAdapterListbox(newtComponent AdapterListbox,pci_device * dev)
{
	int	i = 0;

	newtListboxClear(AdapterListbox);
	
	while(dev)
	{
		char biabuf[16];
		char 	Buffer[70];
		
		sprintf(Buffer,"%2d %c %02d-%02d %-40s %s",
			i,dev->sw_running ? '*' : ' ',
			PCI_BUS(dev->info.busdevfn),
			PCI_SLOT(dev->info.busdevfn & 0xFF),
			dev->cardid==NULL ? "Unknown Madge Token Ring Adapter" : 
			dev->cardid->description,
			madge_bia_as_string(dev->info.bia,biabuf)
		);

		newtListboxAddEntry(AdapterListbox,Buffer,(void *)dev);
		dev = dev->next;
		i++;
	}
}


/*
 * return 	0 = success
 * 			1 = failure
 */
int main(int argc,char **argv)
{
	newtComponent form,listbox,okay,cancel,details,help;
	newtGrid grid,buttons;
	pci_device * dev;
	
	struct newtExitStruct event;
		
	int formdone;
		
	/**********************************************************************/

	newtInit();
	newtCls();
    
#ifdef NEWNEWT
    newtSetSuspendCallback(suspend,NULL);
#else
    newtSetSuspendCallback(suspend);
#endif

	newtDrawRootText(0,0,"MadgeConfig " MADGE_PROBE_VERSION " - (c) 1999 Madge Networks");


	/**********************************************************************/
	
	/* Get the adapter information out of the proc filesystem and driver */

/*  madge_pci_get_adapters_v22(); */
    madge_pci_get_old_running_info();
    madge_pci_get_new_running_info();
    
    dev = madge_pci_get_list();
                        
	if (dev == NULL)
	{
		newtWinMessage("Error","Ok",
				"No Madge Token Ring adapters detected\n"
				"If running on a 2.0.x kernel please insmod the driver\n"
				"before running MadgeConfig\n");
		
		newtFinished(); 
		exit(1);
	}
	
	/*
	 * Display the main adapter list.
	 */

	grid = newtCreateGrid(1,3);

	newtGridSetField(grid,0,0,NEWT_GRID_COMPONENT,
						newtLabel(-1,-1,"IF SW PCI  Description                              BIA"),
						0,0,0,0,NEWT_ANCHOR_LEFT,0);
	
	listbox = newtListbox(-1,-1,3,NEWT_FLAG_SCROLL|NEWT_FLAG_RETURNEXIT);

	FillAdapterListbox(listbox,dev);
		
	newtGridSetField(grid,0,1,NEWT_GRID_COMPONENT,listbox,0,0,2,0,0,0);
	
	buttons = newtButtonBar("Ok",&okay,"Cancel",&cancel,"Details",&details,"Help",&help,NULL);
	
	newtGridSetField(grid,0,2,NEWT_GRID_SUBGRID,buttons,
						0,1,0,0,0,NEWT_GRID_FLAG_GROWX);

	newtListboxSetCurrent(listbox,0);

	form = newtForm(NULL,NULL,0);
	newtFormAddHotKey(form,NEWT_KEY_F1);
	newtGridAddComponentsToForm(grid,form,1);
	newtGridWrappedWindow(grid,"Madge Token Ring Adapters");
	newtGridFree(grid,1);
	
	do {
		pci_device * seldev;

		newtFormRun(form,&event);
		
		if (event.u.key == NEWT_KEY_F12)
			formdone = 1;
			
		if (event.u.co == details || event.u.co == listbox)
		{
			seldev  = (pci_device *)newtListboxGetCurrent(listbox);

			DisplayAdapterInfo(seldev);

			formdone = 0;
		}

		if (event.u.co == help)
		{
			newtWinMessage("Help","Ok",
					"The list box displays all Madge Token Ring adapters found by the program\n"
					"The fields in the list box are as follows:-\n\n"
					"  IF          - The Linux interface number (i.e. tr(N))\n"
					"  SW          - Whether there is currently software running on\n"
					"                that adapter (This will always be the\n"
					"                case for 2.0.x because it is the only way\n"
					"                MadgeConfig can detect the adapter\n"
					"  PCI         - The PCI Bus and Device number of the adapter\n"
					"  Description - Description of the type of adapter\n"
					"  BIA         - The burned in address of the adapter\n"
					"                (not always available)\n");
			formdone = 0;
		}

		if (event.u.co == okay)
		{
			formdone = 1;
			if (argv[1])
				generate_conf_modules(argv[1],dev);
		}

		if (event.u.co == cancel)
		{
			formdone = 1;
		}
		
				
	} while(!formdone);

	newtFormDestroy(form);

	newtPopWindow();
			
	newtFinished();	

	return 0;
}

void generate_conf_modules(char * filename, pci_device * head)
{
	int i = 0,count;
	pci_device * dev;	
	char * pcmcia;
	FILE * fptr;
		
	if (filename == NULL)
	{
		/*
		 * If no file name given don't bother trying to generate the
		 * conf.modules information
		 */
		 
		exit(1);
	}
		
	if (NULL == (fptr = fopen(filename,"w+")))
	{
		printf("Couldn't open temporary conf.modules file\n");
		exit(1);
	}
	
	fprintf(fptr,"## BEGIN MADGE TOKEN RING INFO ##\n");
	fprintf(fptr,"# Automatically inserted by the installation program\n");

	dev = head;
	while(dev)
	{
		fprintf(fptr,"alias tr%d mtok\n",i);
		dev=dev->next;
		i++;
	}
	
	count = i;
	
	fprintf(fptr,"options mtok ");

	pcmcia = getenv("PCMCIA");
	if (pcmcia && (*pcmcia == 'y' || *pcmcia == 'Y'))
		fprintf(fptr," pcmcia=1 ");
		
	fprintf(fptr,"  ");
	fprintf(fptr,"dtr=");

	dev = head; i = 0;
	while(dev)
	{
		fprintf(fptr,"%d",dev->info.dtr);
		dev=dev->next;
		if (i<count - 1)
			fprintf(fptr,",");
		i++;
	}

	fprintf(fptr,"  ");
	fprintf(fptr,"ringspeed=");

	dev = head; i = 0;
	while(dev)
	{
		fprintf(fptr,"%d",dev->info.ringspeed);
		dev=dev->next;
		if (i<count - 1)
			fprintf(fptr,",");
		i++;
	}

	fprintf(fptr,"  ");
	fprintf(fptr,"txslots=");

	dev = head; i = 0;
	while(dev)
	{
		fprintf(fptr,"%d",dev->info.txslots);
		if (i<count - 1)
			fprintf(fptr,",");
		dev=dev->next;
		i++;
	}

	fprintf(fptr,"  ");
	fprintf(fptr,"rxslots=");

	dev = head; i = 0;
	while(dev)
	{
		fprintf(fptr,"%d",dev->info.rxslots);
		if (i<count - 1)
			fprintf(fptr,",");
		dev=dev->next;
		i++;
	}

	fprintf(fptr,"  ");
	fprintf(fptr,"maxframesize=");

	dev = head; i = 0;
	while(dev)
	{
		fprintf(fptr,"%d",dev->info.max_frame_size);
		if (i<count - 1)
			fprintf(fptr,",");
		dev=dev->next;
		i++;
	}

	fprintf(fptr,"\n\n## END MADGE TOKEN RING INFO ##\n\n");
	fclose(fptr);
}

#if 0

VOID_T poke(STRING_T prompt,BYTE_T type,DWORD_T data)
{
	printf(prompt);
	switch(type)
	{
		case PO_TYPE_N:
			break;
		case PO_TYPE_C:
			printf("%c",data&0xFF);
			break;
		case PO_TYPE_B:
			printf("%02X",data &0xFF);
			break;
		case PO_TYPE_W:
			printf("%04X",data & 0xFFFF);
			break;
		case PO_TYPE_D:
			printf("%08X",data);
			break;
		case PO_TYPE_S:
			printf((char *)data);
			break;
		case PO_TYPE_P:
			printf("%p",data);
			break;
		case PO_TYPE_I:
			printf("%d",data);
			break;
		case PO_TYPE_L:
			printf("%d",data);
			break;
	}		

	fflush(stdout);
}
			
#endif
