/********************************************************************
*	THE USE BY YOU OF THIS SOFTWARE IS GOVERNED IN ALL
*	RESPECTS BY THE MADGE NETWORKS LTD OPEN SOURCE LICENSE.		
*	WE ADVISE YOU TO READ THE TERMS OF THE OPEN SOURCE LICENSE
*	WHICH CAN BE FOUND WITHIN THE LICENSE.TXT FILE WHICH IS PART OF
*	THE SOFTWARE OR BY VISITING THE URL
*	http://www.madge.com/Connect/Downloads/Software/.		
*	MADGE NETWORKS LTD RESERVES THE RIGHT TO SEEK REDRESS FOR
*	ANY VIOLATION OF THE LICENSE'S TERMS					
*********************************************************************/

/***************************************************************************
 *                                                                         
 *                                                                         
 *               ioctl.h  -  Definitions and structure for 
 *                           returning information from mtok_do_ioctl
 *                           to user level software.               
 *
 *                                                                         
 *                Copyright (c) Madge Networks Ltd
 *                
 *                CVS Version History:
 *
 *                $Id: mdgioctl.h,v 1.1.1.1 2006/04/27 15:41:46 br Exp $
 *
 ***************************************************************************/

#ifndef __MTOK_IOCTL_H
#define __MTOK_IOCTL_H

/*	Do not use any UDT or Linux defined structures in this file as this would
 *	make the driver IOCTL interface depenedent upon Linux or UDT version
 *  which is not ideal.
 */

/****
 * MTOK_SIOCMDGID 
 *
 * Use this IOCTL to determine if the Token Ring device is controlled by
 * a Madge Driver and to determine the version number of the driver and
 * hence which other IOCTL the driver supports.
 */

#define MTOK_SIOCMDGID			(SIOCDEVPRIVATE + 0)

#define MTOK_ID_IN  0x3a495f27
#define MTOK_ID_OUT 0x45f75721

typedef struct __MTOK_IOCTL_MDGID {
	int len;				/* IN:		Length of the structure */
	int sig;				/* INOUT:	The Madge Signature */
	int version;			/* OUT:		The driver version */
} MTOK_IOCTL_MDGID;


/****
 * MTOK_SIOCADAPTERINFO
 *
 * Use this IOCTL to obtain information and statistics about the adapter.
 */

#define MTOK_SIOCADAPTERINFO	(SIOCDEVPRIVATE + 1)

typedef struct __MTOK_IOCTL_INFO
{
	/* Input Values */

	int len;		 		/* Length of structure */
	int sig;				/* The Madge Signature */
	
	/* Interface Identification */

	char name[9];			/* Device name (e.g. tr0) */

	int type;				/* UDT Type	*/
	int subtype;			/* UDT Subtype */

	int busdevfn;			/* PCI Bus, Device, Function */

	/* Adapter Resources */

	int iobase;
	int irq;
	int xfermode;
	
	int state;

	/* Module parameters passed when loaded */

	int max_frame_size;
	int ringspeed;			/* NB. this is what was passed on cmd line */
	int dtr;
	int rxslots;
	int txslots;

	/* Token Ring Interface Addresses */

	unsigned char bia[6];	/* Burned in address */
	unsigned char laa[6];	/* Current address */
	unsigned char group[6];	/* Group Address */
	unsigned char func[6];	/* Functional Address */
	unsigned char naun[6];	/* NAUN */
	
	int promiscuous_level;	/* Promiscuous mode */
	
	char product_instance_id[18];
	
	/* Open Information */
	
	int error_code;
	int fault_phase;
	int open_ring_speed;
	int max_frame_size_avail;
	int max_rcv_buf_size;
	int mode;

	int ring_status;
	int last_beacon_type;
	int local_ring_number;
	int udi_state;
} MTOK_IOCTL_INFO;

/****
 * MTOK_SIOCADAPTERSTATS
 *
 * Use this IOCTL to obtain information and statistics about the adapter.
 */

#define MTOK_SIOCADAPTERSTATS	(SIOCDEVPRIVATE + 2)

typedef struct __MTOK_IOCTL_STATS
{
	/* Input Values */

	int len;		 		/* Length of structure */
	int sig;				/* The Madge Signature */

	/* Interface Statistics */

	unsigned long rx_packets;
	unsigned long rx_bytes;
	unsigned long rx_errors;
	unsigned long rx_dropped;
		
	unsigned long tx_packets;
	unsigned long tx_bytes;
	unsigned long tx_errors;
	unsigned long tx_dropped;

	/* Token Ring Errors */
	
	unsigned long line_errors;
	unsigned long internal_errors;
	unsigned long burst_errors;
	unsigned long ari_fci_errors;
	unsigned long abort_delimiters;
	unsigned long lost_frame_errors;
	unsigned long congestion_errors;
	unsigned long frame_copied_errors;
	unsigned long frequency_errors;
	unsigned long token_errors;
	unsigned long dma_bus_errors;
	unsigned long dma_parity_errors;
} MTOK_IOCTL_STATS;

/****
 * MTOK_SIOCPRI
 *
 * Use this IOCTL to set up priority structures.
 */

#define MTOK_SIOCGPRI	(SIOCDEVPRIVATE + 3)
#define MTOK_SIOCSPRI	(SIOCDEVPRIVATE + 4)

typedef struct {
	/* Input Values */

	int len;		 		/* Length of structure */
	int sig;				/* The Madge Signature */

} MTOK_PRIORITY_IOCTL;

#define MTOK_SIOCGFILTERMAC	(SIOCDEVPRIVATE + 5)
#define MTOK_SIOCSFILTERMAC	(SIOCDEVPRIVATE + 6)

typedef struct {
	/* Input Values */

	int len;		 		/* Length of structure */
	int sig;				/* The Madge Signature */

	int wantmac;
	
} MTOK_IOCTL_FILTERMAC;

#endif /*  __MTOK_IOCTL_H */



