/********************************************************************
*	THE USE BY YOU OF THIS SOFTWARE IS GOVERNED IN ALL
*	RESPECTS BY THE MADGE NETWORKS LTD OPEN SOURCE LICENSE.		
*	WE ADVISE YOU TO READ THE TERMS OF THE OPEN SOURCE LICENSE
*	WHICH CAN BE FOUND WITHIN THE LICENSE.TXT FILE WHICH IS PART OF
*	THE SOFTWARE OR BY VISITING THE URL
*	http://www.madge.com/Connect/Downloads/Software/.		
*	MADGE NETWORKS LTD RESERVES THE RIGHT TO SEEK REDRESS FOR
*	ANY VIOLATION OF THE LICENSE'S TERMS					
*********************************************************************/

/***************************************************************************
 *                                                                         
 *	diag.c   - Very simple console app to query
 *             a Madge Token Ring adapter
 *               
 *	1.1		Tom Gallagher (Madge)
 *  1.0     Joe Marshall (Madge) 13.7.99                   
 *                                                                         
 *  Copyright (c) Madge Networks Ltd 1999                    
 *
 *
 * Notes:
 *	 The byte counters will always be zero for Linux 2.0.x
 *
 *   This program can be used to set the driver to pass up MAC frames
 *
 *   It is a very simple ifconfig but for specific Token Ring functionality.
 *
 ***************************************************************************/

#include <stdio.h>
#include <fcntl.h>
#include <linux/types.h>
#include <sys/types.h>
#include <sys/ioctl.h>
#include <net/if.h>
#include <linux/pci.h>
#include <errno.h>
#include <netdb.h>
#include <string.h>

#include "madgetr.h" 
#include "mdgioctl.h"

#define VERSION	"1.1"

char * CardNames[] = CARD_NAME_STRINGS;

#ifndef PCI_SLOT
#define PCI_SLOT(devfn)         (((devfn) >> 3) & 0x1f)
#define PCI_FUNC(devfn)         ((devfn) & 0x07)
#endif

void usage()
{
	printf(	
			 "Usage: trconfig [device [[-]mac]]\n");
}

void display_address(unsigned char *addr)
{
	int c;
	for(c=0;c<SIZEOF_MAC_ADDRESS;c++)
	{
    	printf("%02X",addr[c]);
	}
}

void display_stats(	int version,
					MTOK_IOCTL_STATS *stats,
					MTOK_IOCTL_INFO  *info)
{
	int c;

	printf("Device:              %s\n",info->name);
	printf("Driver version:      %d.%d.%d\n",	(version & 0xFF0000) >> 16,
												(version & 0x00FF00) >> 8,
												(version & 0x0000FF) );



	printf("Driver ID:           %s\n\n",
		info->product_instance_id);  

	printf("Card:                %s\n",CardNames[info->subtype]);
	if (info->busdevfn)
	{
		printf("PCI Info:            Bus %d Device %d Function %d\n",
								(info->busdevfn & 0xFF00) >> 16,
								PCI_SLOT((info->busdevfn & 0xFF)),
								PCI_FUNC((info->busdevfn & 0xFF))  );
	}
	printf("IO Base:             %X\n",info->iobase);
	printf("IRQ:                 %i\n",info->irq);
	printf("\n");
	
	printf("Max Frame Size:      %d\n",info->max_frame_size);
	printf("RX Slots:            %d\n",info->rxslots);
	printf("TX Slots:            %d\n",info->txslots);
	printf("\n");
	
	printf("Burnt in Address:    ");
	display_address(info->bia);
	printf("\n");

	printf("LAA:                 ");
	display_address(info->laa);
	printf("\n");
	
	printf("Upstream Neighbour:  ");
	display_address(info->naun);
	printf("\n");
	

	printf("Current group:       ");
   	display_address(info->group);
	printf("\n");

	printf("Open Ring Speed:     ");
	switch(info->open_ring_speed)
	{
		case RS_AUTO:	printf("Automatic\n");break;
		case RS_ADAPTER:printf("Unknown until open\n");break;
		case RS_4M:		printf("4 Mbits/s\n");break;
		case RS_16M:	printf("16 Mbits/s\n");break;
		case RS_100M:	printf("100 Mbits/s\n");break;
		case RS_UNKNOWN:
		default:
						printf("Unknown\n");break;
    }

	printf("Connection Type:     ");
	switch(info->mode)
	{
		case OPEN_MODE_CLASSIC:	printf("Classic\n");break;
		case OPEN_MODE_SINGLE:	printf("Single station\n");break;
		case OPEN_MODE_DTR:		printf("DTR\n");break;
		default:
		case OPEN_MODE_UNKNOWN:	printf("Unknown\n");break;
    }

	printf("Promiscuous level:   ");
	switch(info->promiscuous_level)
    {
    	case LLC:				printf("LLC frames for this address\n");break;
    	case LLC_AND_MAC:		printf("LLC and MAC frames for this address\n");break;
    	case ALL_LLC_AND_MAC:	printf("All LLC and MAC frames\n");break;
    	case ALL_LLC:			printf("All LLC frames\n");break;
    	default:
    		printf("Unknown\n");break;
    }

	printf("Last Beacon Type:    0x%x\n",info->last_beacon_type);
	printf("Local Ring Number:   0x%x\n",info->local_ring_number);

	printf("State:               Device ");
	switch(info->udi_state)
    {
		case REGISTERED:printf("registered");break;
    	case COMMIT_PENDING:printf("setting up hardware");break;
    	case CONFIGURED:printf("hardware set up");break;
    	case READY:printf("ready");break;
    	case OPEN_PENDING:printf("opening");break;
    	case OPEN:printf("open");break;
    	case CLOSE_PENDING:printf("closing");break;
    }

	switch(info->state)
    {
		case MTOK_STATE_UNKNOWN:	printf(" (State unknown)\n");break;
		case MTOK_STATE_DEAD:		printf(" (Adapter dead)\n");break;
    	case MTOK_STATE_REMOVED:	printf(" (Adapter removed)\n");break;
		case MTOK_STATE_INITIALISING:printf(" (Adapter initialising)\n");break;
		case MTOK_STATE_READY:		printf(" (Adapter ready)\n");break;
		case MTOK_STATE_RUNNING:	printf(" (Adapter running)\n");break;
	    case MTOK_STATE_CLOSING:	printf(" (Adapter closing)\n");break;
	    case MTOK_STATE_CLOSED:		printf(" (Adapter closed)\n");break;
	    case MTOK_STATE_OPENING:	printf(" (Adapter opening)\n");break;
    	case MTOK_STATE_RE_OPENING:	printf(" (Adapter reopening)\n");break;
    	case MTOK_STATE_OPEN:		printf(" (Adapter open)\n");break;
    }

  printf("Ring Status:         ");
  if(info->ring_status & RING_STATUS_RING_RECOVERY)
    printf("Ring recovery mode\n                   ");
  if(info->ring_status & RING_STATUS_SINGLE_STATION)
    printf("Single station mode\n                   ");
  if(info->ring_status & RING_STATUS_COUNTER_OVERFLOW)
    printf("Counter overflow\n                   ");
  if(info->ring_status & RING_STATUS_REMOVE_RECEIVED)
    printf("Remove received\n                   ");
  if(info->ring_status & RING_STATUS_AUTO_REMOVAL)
    printf("Auto removal\n                   ");
  if(info->ring_status & RING_STATUS_LOBE_FAULT)
    printf("Lobe fault\n                   ");
  if(info->ring_status & RING_STATUS_TX_BEACON)
    printf("Transmit beacon\n                   ");
  if(info->ring_status & RING_STATUS_SOFT_ERROR)
    printf("Soft error\n                   ");
  if(info->ring_status & RING_STATUS_HARD_ERROR)
    printf("Hard error\n                   ");
  if(info->ring_status & RING_STATUS_SIGNAL_LOSS)
    printf("Signal loss\n                   ");
  if(info->ring_status==0)
    printf("Normal\n                   ");

  printf("\nCounters\n");

  printf("    %10s  %10s  %10s  %10s\n","Bytes","Frames","Dropped","Errors");

  printf("RX  %10i  %10i  %10i  %10i\n",
	 stats->rx_bytes,
	 stats->rx_packets,
	 stats->rx_dropped,
	 stats->rx_errors);

  printf("TX  %10i  %10i  %10i  %10i\n",
	 stats->tx_bytes,
	 stats->tx_packets,
	 stats->tx_dropped,
	 stats->tx_errors);

  printf("\nErrors\n");

  printf("      Line:%i\t   Internal:%i\t     Burst: %i\n",
	 stats->line_errors,
	 stats->internal_errors,
	 stats->burst_errors);

  printf("   ARI/FCI:%i\tAbort Delim:%i\tLost Frame: %i\n",
	 stats->ari_fci_errors,
	 stats->abort_delimiters,
	 stats->lost_frame_errors);

  printf("Congestion:%i\t Frame Copy:%i\t Frequency: %i\n",
	 stats->congestion_errors,
	 stats->frame_copied_errors,
	 stats->frequency_errors);

  printf("     Token:%i\t    DMA Bus:%i\tDMA Parity: %i\n",
	 stats->token_errors,
	 stats->dma_bus_errors,
	 stats->dma_parity_errors);

	printf("\n\n");
}

/*
 * Returns:-
 *		0 - Success
 *		1 - Fail but carry on looking
 *		2 - Fail and stop searching other TR devices
 */
int display_one_adapter(int s,char * device)
{
		struct ifreq ifr;
		MTOK_IOCTL_STATS stats;
		MTOK_IOCTL_INFO	 info;
		MTOK_IOCTL_MDGID mdgid;

	
		/* Firstly, establish if this device is controlled
		   by a Madge driver and which version number if so
		 */

		strcpy(ifr.ifr_name,device);

		ifr.ifr_data = (void *)&mdgid;
		mdgid.len = sizeof(MTOK_IOCTL_MDGID);
		mdgid.sig = MTOK_ID_IN;

		if (ioctl(s,MTOK_SIOCMDGID,&ifr) < 0)
		{
			if (errno == ENODEV)
			{
				/* Error no device - reached the end of the line */
				return 2;
			}
			if (errno == EINVAL)
			{
				/* We just queried a TR device which doesn't support private IOCTLs */
				return 1;
			}
			/* A proper error */
			perror("MTOK_SIOCMDGID");
			return 2;
		}
	
		if (mdgid.sig != MTOK_ID_OUT)
		{
			/* Not a Madge Device - continue looping there may be others */
			return 1;
		}
	
		/*
		 * Now query the driver for statistics and information
		 */
	 
		ifr.ifr_data = (void *)&info;
		info.len = sizeof(MTOK_IOCTL_INFO);
		info.sig = MTOK_ID_IN;
	
		if (ioctl(s,MTOK_SIOCADAPTERINFO,&ifr)<0)
		{
			/* Some sort of error in the querying the device - 
			 * still need to carry on looking so return 1
			 */
			perror("MTOK_SIOCADAPTERINFO");
			return 1;
		}

		ifr.ifr_data = (void *)&stats;
		stats.len = sizeof(MTOK_IOCTL_STATS);
		stats.sig = MTOK_ID_IN;
	
		if (ioctl(s,MTOK_SIOCADAPTERSTATS,&ifr)<0)
		{
			/* Some sort of error in the querying the device - 
			 * still need to carry on looking so return 1
			 */
			perror("MTOK_SIOCADAPTERSTATS");
			return 1;
		}		
		
		display_stats(mdgid.version,&stats,&info);
}

int set_want_mac(int s, char * device, int wantmac)
{
	struct ifreq ifr;
	MTOK_IOCTL_FILTERMAC filtermac;
	MTOK_IOCTL_MDGID mdgid;

	/* Firstly, establish if this device is controlled
	   by a Madge driver and which version number if so
	 */

	strcpy(ifr.ifr_name,device);

	ifr.ifr_data = (void *)&mdgid;
	mdgid.len = sizeof(MTOK_IOCTL_MDGID);
	mdgid.sig = MTOK_ID_IN;

	if (ioctl(s,MTOK_SIOCMDGID,&ifr) < 0)
	{
		if (errno == ENODEV)
		{
			/* Error no device - reached the end of the line */
			return 2;
		}
		if (errno == EINVAL)
		{
			/* We just queried a TR device which doesn't support private IOCTLs */
			return 1;
		}
		/* A proper error */
		perror("MTOK_SIOCMDGID");
		return 2;
	}
	
	if (mdgid.sig != MTOK_ID_OUT)
	{
		/* Not a Madge Device - continue looping there may be others */
		return 1;
	}

	ifr.ifr_data = (void *)&filtermac;
	filtermac.len = sizeof(MTOK_IOCTL_FILTERMAC);
	filtermac.sig = MTOK_ID_IN;

	filtermac.wantmac = wantmac;
	
	if (ioctl(s,MTOK_SIOCSFILTERMAC,&ifr) <0)
	{
		perror("MTOK_SIOCSFILTERMAC");
		return 0;
	}
	return 1;
}

int main(int argc,char **argv)
{
	int s,i;
	int rval=0;
	char device[8];
	char banner[] = "Madge Diagnostics v" VERSION "\nCopyright (c) 1999 Madge Networks Ltd\n\n";

	printf(banner);

	if (argc > 3)
	{
		usage();
		return 1;
	}
	
	if ((s = socket(AF_INET,SOCK_DGRAM,0)) < 0)
	{
 		perror("Error creating socket");
  		return 1;
	}

	
	if (argc == 2)
	{
		/*
		 * Mode 1: Display for one specified device only
		 */
		rval = display_one_adapter(s,argv[1]);
		if (rval == 1)
			fprintf(stderr,"Error occurred or not a Madge Token Ring device\n");
	}
	else if(argc==3)
	{
		/*
		 * Turn on/off MAC frame reception
		 */
		 if (strcmp(argv[2],"-mac")==0)
		 {
			set_want_mac(s,argv[1],0);
		 }
		 else
		 if (strcmp(argv[2],"mac")==0)
		 {
			set_want_mac(s,argv[1],1);
		 }
		 else
		 {
		 	usage();
		 }
	} 
	else
	{
		/*
		 * Mode 2: Display info for all devices
		 */
		i = 0;
		do
		{
			sprintf(device,"tr%d",i);
			rval = display_one_adapter(s,device);
			i++;
		} while(rval == 0 || rval==1);
	}
	close(s);
	return 0;
}
